@@ -1,49 +1,53 @@
 # -*- coding: utf-8 -*-
 """Track Earth satellites given TLE data, using up-to-date 2020 SGP4 routines.
 
-This Python package computes the position and velocity of an
-earth-orbiting satellite, given the satellite's TLE orbital elements
-from a source like `CelesTrak <https://celestrak.com/>`_.  It implements
-the most recent version of SGP4, and is regularly run against the SGP4
-test suite to make sure that its satellite position predictions **agree
-to within 0.1 mm** with the predictions of the standard distribution of
-the algorithm.  This error is far less than the 1–3 km/day by which
-satellites themselves deviate from the ideal orbits described in TLE
-files.
+This package compiles the official C++ code from `Revisiting Spacetrack
+Report #3 <https://celestrak.org/publications/AIAA/2006-6753/>`_ (AIAA
+2006-6753) and uses it to compute the positions of satellites in Earth
+orbit.  Orbital elements can be read from either a legacy TLE file or
+from a modern OMM element set, both of which you can fetch from a site
+like `CelesTrak <https://celestrak.com/>`_.
 
-* If your platform supports it, this package compiles and uses the
-  verbatim source code from the official C++ version of SGP4.
+If your machine can’t install or compile the C++ code, then this package
+falls back to using a slower pure-Python implementation of SGP4.  Tests
+make sure that its positions **agree to within 0.1 mm** with the
+standard version of the algorithm — an error far less than the
+1–3 km/day by which satellites themselves deviate from the ideal orbits
+described in TLE files.
 
-* Otherwise, a slower but reliable Python implementation of SGP4 is used
-  instead.
-
-* If, instead of asking for the position of a single satellite at a
-  single time, you supply this library with an array of satellites and
-  an array of times, then the arrays can be processed using machine code
-  instead of requiring you to run a slow Python loop over them.
+An accelerated routine is available that, given a series of times and of
+satellites, computes a whole array of output positions using a fast C++
+loop.  See the “Array Acceleration” section below.
 
 Note that the SGP4 propagator returns raw *x,y,z* Cartesian coordinates
 in a “True Equator Mean Equinox” (TEME) reference frame that’s centered
 on the Earth but does not rotate with it — an “Earth centered inertial”
 (ECI) reference frame.  The SGP4 propagator itself does not implement
 the math to convert these positions into more official ECI frames like
-J2000 or the ICRF; nor to convert positions into any Earth-centered
-Earth-fixed (ECEF) frames like the ITRS; nor to convert them to
-latitudes and longitudes through an Earth ellipsoid like WGS84.
-
-For conversions into other coordinate frames, look for a comprehensive
-astronomy library that is built atop this one, like the `Skyfield
-<https://rhodesmill.org/skyfield/>`_ library:
-
-https://rhodesmill.org/skyfield/earth-satellites.html
+J2000 or the ICRS, nor into any Earth-centered Earth-fixed (ECEF) frames
+like the ITRS, nor into latitudes and longitudes through an Earth
+ellipsoid like WGS84.  For conversions into these other coordinate
+frames, look for a comprehensive astronomy library, like the `Skyfield
+<https://rhodesmill.org/skyfield/>`_ library that is built atop this one
+(see the `section on Earth satellites
+<https://rhodesmill.org/skyfield/earth-satellites.html>`_ in its
+documentation).
 
 Usage
 -----
 
+You will probably first want to to check whether your machine has
+successfully installed the fast SGP4 C++ code, or is using the slow
+Python version (in which case this value will be false):
+
+>>> from sgp4.api import accelerated
+>>> print(accelerated)
+True
+
 This library uses the same function names as the official C++ code, to
-help users who may already be familiar with SGP4 in other languages.
-Here is how to compute the x,y,z position and velocity for the
-International Space Station at 12:50:19 on 29 June 2000:
+help users who are already familiar with SGP4 in other languages.  Here
+is how to compute the x,y,z position and velocity for the International
+Space Station at 12:50:19 on 29 June 2000:
 
 >>> from sgp4.api import Satrec
 >>>
@@ -109,15 +113,30 @@ compute ``jd`` and ``fr`` from calendar dates using ``jday()``.
 >>> fr
 0.5
 
-OMM
----
+Double-checking your TLE lines
+------------------------------
 
-The industry is making adjustments because the fixed-width TLE format
+Because TLE is an old punch-card fixed-width format, it’s very sensitive
+to whether exactly the right number of spaces are positioned in exactly
+the right columns.  If you suspect that your satellite elements aren’t
+getting loaded correctly, try calling the slow pure-Python version of
+``twoline2rv()``, which performs extra checks that the fast C++ doesn’t:
+
+>>> from sgp4.earth_gravity import wgs72
+>>> from sgp4.io import twoline2rv
+>>> assert twoline2rv(s, t, wgs72)
+
+Any TLE formatting errors will be raised as a ``ValueError``.
+
+Using OMM elements instead of TLE
+---------------------------------
+
+The industry is migrating away from the original TLE format, because it
 will soon run out of satellite numbers.
 
 * Some TLE files now use a new “Alpha-5” convention that expands the
   range of satellite numbers by using an initial letter; for example,
-  “E8493” means satellite 148493.  This library now supports the Alpha-5
+  “E8493” means satellite 148493.  This library supports the Alpha-5
   convention and should return the correct integer in Python.
 
 * Some authorities are now distributing satellite elements in an “OMM”
@@ -210,8 +229,18 @@ datetime.datetime(2019, 12, 9, 16, 38, 29, 363423, tzinfo=UTC)
 Array Acceleration
 ------------------
 
-To avoid the expense of Python loops when you have many dates, you can
-pass them as arrays to another method that understands NumPy:
+To avoid the expense of running a Python loop when you have many dates
+and times for which you want a position, you can pass your Julian dates
+as arrays.  The array routine is only faster if your machine has
+successfully installed or compiled the SGP4 C++ code, so you might want
+to check first:
+
+>>> from sgp4.api import accelerated
+>>> print(accelerated)
+True
+
+To call the array routine, make NumPy arrays for ``jd`` and ``fr`` that
+are the same length:
 
 >>> import numpy as np
 >>> np.set_printoptions(precision=2)
@@ -357,8 +386,8 @@ first code example above:
 ...     25544,                # satnum: Satellite number
 ...     25545.69339541,       # epoch: days since 1949 December 31 00:00 UT
 ...     3.8792e-05,           # bstar: drag coefficient (1/earth radii)
-...     0.0,                  # ndot: ballistic coefficient (revs/day)
-...     0.0,                  # nddot: mean motion 2nd derivative (revs/day^3)
+...     0.0,                  # ndot: ballistic coefficient (radians/minute^2)
+...     0.0,                  # nddot: mean motion 2nd derivative (radians/minute^3)
 ...     0.0007417,            # ecco: eccentricity
 ...     0.3083420829620822,   # argpo: argument of perigee (radians)
 ...     0.9013560935706996,   # inclo: inclination (radians)
@@ -400,7 +429,8 @@ underlying C++ SGP4 record.  They fall into the following categories.
 These are copied directly from the TLE record but aren’t used by the
 propagation math.
 
-| ``satnum`` — Unique number assigned to the satellite.
+| ``satnum_str`` — Satellite number, as a 5-character string.
+| ``satnum`` — Satellite number, converted to an integer.
 | ``classification`` — ``'U'``, ``'C'``, or ``'S'``
   indicating the element set is Unclassified, Classified, or Secret.
 | ``ephtype`` — Integer “ephemeris type”, used internally by space
@@ -585,6 +615,23 @@ https://pypi.org/project/sgp4/1.4/
 Changelog
 ---------
 
+2023-04-27 — 2.22
+
+* Added a ``satnum_str`` attribute, exposing the fact that the C++ now
+  stores the satellite number as a string; and check that ``satnum`` is
+  never greater than 339999.
+
+* Fixed the units of the ``nddot`` attribute when the value is loaded
+  from an OMM record.  (Since the TLE computation itself ignores this
+  attribute, this did not affect any satellite positions.)
+
+* Enhanced the fallback Python version of ``twoline2rv()`` to verify
+  that TLE lines are ASCII, and added documentation using it to
+  double-check TLEs that might suffer from non-ASCII characters.
+
+* If the user doesn’t set a satellite’s ``classification``, it now
+  defaults to ``'U'`` for ‘unclassified’.
+
 2022-04-06 — 2.21
 
 * Added ``dump_satrec()`` to the ``sgp4.conveniences`` module.
@@ -655,4 +702,4 @@ Changelog
 | 2012-08-27 — 1.0 — Initial release
 
 """
-__version__ = '2.21'
+__version__ = '2.22'
